{ **********************************************************************
  *                        Program SYSEQ_GJ.PAS                        *
  *                            Version 1.0                             *
  *                    (c) J. Debord, February 2003                    *
  **********************************************************************
  This program solves a system of linear equations (A * X = B) with
  several constant vectors by the Gauss-Jordan method. The system is
  stored in a data file with the following structure :

    Line  1            : Number of rows (N) and columns (M)
    Lines 2 to (N + 1) : Matrices, A in the first N columns,
                         B in the last (M - N) columns

  The file MATRIX2.DAT is an example data file with N = 4 and M = 9
  ********************************************************************** }

program SysEq_GJ;

uses
  FMath, Matrices;

  procedure WriteMatrix(Title : String; A : PMatrix; N, Col1, Col2 : Integer);
{ ----------------------------------------------------------------------
  Writes a matrix A (N x M) from Col1 to Col2
  ---------------------------------------------------------------------- }
  var
    I, J : Integer;
  begin
    WriteLn(Title, ' :');
    WriteLn;
    for I := 1 to N do
      begin
        for J := Col1 to Col2 do
          Write(A^[I]^[J]:12:6);
        WriteLn;
      end;
    WriteLn;
  end;

var
  A    : PMatrix;  { System matrix }
  N, M : Integer;  { Matrix dimensions }
  D    : Float;    { Determinant }
  F    : Text;     { Data file }
  I, J : Integer;  { Loop variables }

begin
  { Read matrix from file }
  Assign(F, 'matrix2.dat');
  Reset(F);
  Read(F, N, M);
  DimMatrix(A, N, M);
  for I := 1 to N do
    for J := 1 to M do
      Read(F, A^[I]^[J]);
  Close(F);

  { Read and display data }
  WriteMatrix('System matrix', A, N, 1, N);
  WriteMatrix('Constant vectors', A, N, N+1, M);

  { Solve system }
  case GaussJordan(A, 1, N, M, D) of
    MAT_OK     : WriteMatrix('Solution vectors', A, N, N+1, M);
    MAT_SINGUL : Write('Singular matrix!');
  end;
end.

