{ **********************************************************************
  *                        File REG_PLOT.INC                           *
  *                           Version 1.7                              *
  *                   (c) J. Debord, January 2004                      *
  **********************************************************************
   Include file used by several linear or nonlinear regression programs
  ********************************************************************** }

  {$F+}
  function PlotRegFunc(X : Float) : Float;
{ ----------------------------------------------------------------------
  Defines the function to be plotted
  ---------------------------------------------------------------------- }
  begin
    PlotRegFunc := RegFunc(X, B);
  end;
  {$F-}

  procedure PlotGraph(Title : String;
                      X, Y  : PVector;
                      N     : Integer);
{ ----------------------------------------------------------------------
  Plots the regression function
  ---------------------------------------------------------------------- }
  begin
    GraphTitle.Text := Title;
    XTitle.Text := XName;
    YTitle.Text := YName;

    { Determine scale automatically }
    AutoScale(X, 1, N, XAxis);
    AutoScale(Y, 1, N, YAxis);

    { Plot graph }
    if GraphOk then
      begin
        WriteLegend(0);
        SetClipping(True);

        { Plot points }
        PlotCurve(X, Y, 1, N, 1, 0);

        { Plot fitted function }
        PlotFunc(PlotRegFunc, XAxis.Min, XAxis.Max, 1);

        { Leave graphic mode }
        Readln;
        CloseGraph;
      end;
  end;

  procedure SaveGraph(Title : String;
                      X, Y  : PVector;
                      N     : Integer);
  var
    F : Text;
  begin
    { Set window limits }
    Xwin1 := 10; Ywin1 := 10;
    Xwin2 := 90; Ywin2 := 90;

    { Set graph titles }
    XTitle.Text := XName;
    YTitle.Text := YName;

    InitTexGraph(F, 'figure.tex');
    WriteXAxis(F);
    WriteYAxis(F);

    WritePoints(F, X, Y, 1, N, 1, 2);
    WriteFunc(F, PlotRegFunc, XAxis.Min, XAxis.Max, 100, 5, 'solid', False);

    WriteLn(F, '\end{pspicture}'#10);
    WriteLn(F);

    Close(F);
  end;

